<?php
/**
 *   @copyright Copyright (c) 2012 Quality Unit s.r.o.
 *   @package GwtPhpFramework
 *   @author Juraj Simon
 *
 *   Licensed under the Quality Unit, s.r.o. Standard End User License Agreement,
 *   Version 1.0 (the "License"); you may not use this file except in compliance
 *   with the License. You may obtain a copy of the License at
 *   http://www.postaffiliatepro.com/licenses/license
 */

class Gpf_Currency_Helper extends Gpf_Object {

    /**
     * daily currencies : http://www.ecb.europa.eu/stats/eurofxref/eurofxref-daily.xml
     * historical records (very big): http://www.ecb.europa.eu/stats/eurofxref/eurofxref-hist.xml
     */
    private function loadCurrencyRate($sourceCurrency, $targetcurrency, $date, $type = Gpf_Db_CurrencyRate::TYPE_DAILY) {
        $sql = new Gpf_SqlBuilder_SelectBuilder();
        $sql->select->addAll(Gpf_Db_Table_CurrencyRates::getInstance());
        $sql->from->add(Gpf_Db_Table_CurrencyRates::getName());
        $sql->where->add(Gpf_Db_Table_CurrencyRates::SOURCE_CURRENCY, '=', $sourceCurrency);
        $sql->where->add(Gpf_Db_Table_CurrencyRates::TARGET_CURRENCY, '=', $targetcurrency);
        $sql->where->add(Gpf_Db_Table_CurrencyRates::VALID_FROM, '<=', $date);
        $sql->where->add(Gpf_Db_Table_CurrencyRates::VALID_TO, '>=', $date);
        $sql->where->add(Gpf_Db_Table_CurrencyRates::TYPE, '=', $type);
        Gpf_Log::debug($sql->toString());
        return $sql->getOneRow()->get(Gpf_Db_Table_CurrencyRates::RATE);
    }
    
    
    public function getCurrentAvgEurRate($targetcurrency, Gpf_DateTime_Range $range) {
        $sql = new Gpf_SqlBuilder_SelectBuilder();
        $sql->select->add('avg('.Gpf_Db_Table_CurrencyRates::RATE.')', Gpf_Db_Table_CurrencyRates::RATE);
        $sql->from->add(Gpf_Db_Table_CurrencyRates::getName());
        $sql->where->add(Gpf_Db_Table_CurrencyRates::SOURCE_CURRENCY, '=', 'EUR');
        $sql->where->add(Gpf_Db_Table_CurrencyRates::TARGET_CURRENCY, '=', $targetcurrency);
        $sql->where->add(Gpf_Db_Table_CurrencyRates::TYPE, '=', Gpf_Db_CurrencyRate::TYPE_DAILY);
        
        $dateCondition = new Gpf_SqlBuilder_CompoundWhereCondition();
        
        $coumpoundCondWithin = new Gpf_SqlBuilder_CompoundWhereCondition();
        $coumpoundCondWithin->add(Gpf_Db_Table_CurrencyRates::VALID_FROM, '>=', $range->getFrom()->toDateTime());
        $coumpoundCondWithin->add(Gpf_Db_Table_CurrencyRates::VALID_TO, '<=', $range->getTo()->toDateTime());
        $dateCondition->addCondition($coumpoundCondWithin, 'OR');
        $coumpoundCondBefore = new Gpf_SqlBuilder_CompoundWhereCondition();
        $coumpoundCondBefore->add(Gpf_Db_Table_CurrencyRates::VALID_FROM, '<=', $range->getFrom()->toDateTime());
        $coumpoundCondBefore->add(Gpf_Db_Table_CurrencyRates::VALID_TO, '>=', $range->getFrom()->toDateTime());
        $dateCondition->addCondition($coumpoundCondBefore, 'OR');
        $coumpoundCondAfter = new Gpf_SqlBuilder_CompoundWhereCondition();
        $coumpoundCondAfter->add(Gpf_Db_Table_CurrencyRates::VALID_FROM, '<=', $range->getTo()->toDateTime());
        $coumpoundCondAfter->add(Gpf_Db_Table_CurrencyRates::VALID_TO, '>=', $range->getTo()->toDateTime());
        $dateCondition->addCondition($coumpoundCondAfter, 'OR');
        
        $sql->where->addCondition($dateCondition);
        Gpf_Log::debug($sql->toString());
        Gpf_Log::debug('Avg rate: ' . $sql->getOneRow()->get(Gpf_Db_Table_CurrencyRates::RATE));
        return $sql->getOneRow()->get(Gpf_Db_Table_CurrencyRates::RATE);
    }
    
    public function getCurrentEurRate($targetcurrency) {
        $today = new Gpf_DateTime();
        Gpf_Log::debug('Getting currency for ' . $targetcurrency . ' on ' . $today->toDate());
        try {
            return $this->loadCurrencyRate('EUR', $targetcurrency, $today->toDateTime());
        } catch (Gpf_DbEngine_NoRowException $e) {
            Gpf_Log::debug('Rate for ' . $targetcurrency . ' in ' . $today->toDate() . ' not found, loading latest...');
        }
        $rawXmlData = file_get_contents('http://www.ecb.europa.eu/stats/eurofxref/eurofxref-daily.xml');
        $xml = new SimpleXMLElement($rawXmlData);
        $attrs = $xml->{"Cube"}->{"Cube"}->attributes();
        $time = $attrs['time'];
        foreach ($xml->{"Cube"}->{"Cube"}->{'Cube'} as $dailyRate) {
            $rateSettings = $dailyRate->attributes();
            Gpf_Log::debug('Currency ' . $rateSettings['currency'] . ' loaded from XML');
            if ($rateSettings['currency'] == $targetcurrency) {
                $this->saveDailyRate($today, $targetcurrency, (Double)$rateSettings['rate']);
                return (Double)$rateSettings['rate'];
            }
        }
    }

    /**
     * Get latest conversion rate between 2 currencies
     * @param string $currencyFrom string e.g. EUR, USD
     * @param string $currencyTo string e.g. EUR, USD
     * @return double conversion ratio between two currencies, in case of any error false is returned.
     */
    public function getCurrencyRate($currencyFrom, $currencyTo) {
    	$currencyFrom = strtoupper($currencyFrom);
    	$currencyTo = strtoupper($currencyTo);
    	
    	if (!$this->isCurrencySupported($currencyFrom) || !$this->isCurrencySupported($currencyTo)) {
    		//symbol is not supported
    		return false;
    	}
    	
    	$request = new Gpf_Net_Http_Request();
    	$request->setMethod('GET');
    	$request->setUrl('http://www.webservicex.net/CurrencyConvertor.asmx/ConversionRate?FromCurrency=' . $currencyFrom . '&ToCurrency=' . $currencyTo);
    		
    	$client = new Gpf_Net_Http_Client();
    	try {
    		$input = $client->execute($request)->getBody();
    		return trim(strip_tags($input));
    	} catch (Exception $e) {
    		return false;
    	}
    	return false;
    }
    
    private function isCurrencySupported($currencySymbol) {
    	$supported = $this->getSupportedCurrencies();
    	return isset($supported[$currencySymbol]);
    }
    
    private function getSupportedCurrencies() {
    	return array (
    	'AFA'=>'Afghanistan Afghani',
    	'ALL'=>'Albanian Lek',
    	'DZD'=>'Algerian Dinar',
    	'ARS'=>'Argentine Peso',
    	'AWG'=>'Aruba Florin',
    	'AUD'=>'Australian Dollar',
    	'BSD'=>'Bahamian Dollar',
    	'BHD'=>'Bahraini Dinar',
    	'BDT'=>'Bangladesh Taka',
    	'BBD'=>'Barbados Dollar',
    	'BZD'=>'Belize Dollar',
    	'BMD'=>'Bermuda Dollar',
    	'BTN'=>'Bhutan Ngultrum',
    	'BOB'=>'Bolivian Boliviano',
    	'BWP'=>'Botswana Pula',
    	'BRL'=>'Brazilian Real',
    	'GBP'=>'British Pound',
    	'BND'=>'Brunei Dollar',
    	'BIF'=>'Burundi Franc',
    	'XOF'=>'CFA Franc (BCEAO)',
    	'XAF'=>'CFA Franc (BEAC)',
    	'KHR'=>'Cambodia Riel',
    	'CAD'=>'Canadian Dollar',
    	'CVE'=>'Cape Verde Escudo',
    	'KYD'=>'Cayman Islands Dollar',
    	'CLP'=>'Chilean Peso',
    	'CNY'=>'Chinese Yuan',
    	'COP'=>'Colombian Peso',
    	'KMF'=>'Comoros Franc',
    	'CRC'=>'Costa Rica Colon',
    	'HRK'=>'Croatian Kuna',
    	'CUP'=>'Cuban Peso',
    	'CYP'=>'Cyprus Pound',
    	'CZK'=>'Czech Koruna',
    	'DKK'=>'Danish Krone',
    	'DJF'=>'Dijibouti Franc',
    	'DOP'=>'Dominican Peso',
    	'XCD'=>'East Caribbean Dollar',
    	'EGP'=>'Egyptian Pound',
    	'SVC'=>'El Salvador Colon',
    	'EEK'=>'Estonian Kroon',
    	'ETB'=>'Ethiopian Birr',
    	'EUR'=>'Euro',
    	'FKP'=>'Falkland Islands Pound',
    	'GMD'=>'Gambian Dalasi',
    	'GHC'=>'Ghanian Cedi',
    	'GIP'=>'Gibraltar Pound',
    	'XAU'=>'Gold Ounces',
    	'GTQ'=>'Guatemala Quetzal',
    	'GNF'=>'Guinea Franc',
    	'GYD'=>'Guyana Dollar',
    	'HTG'=>'Haiti Gourde',
    	'HNL'=>'Honduras Lempira',
    	'HKD'=>'Hong Kong Dollar',
    	'HUF'=>'Hungarian Forint',
    	'ISK'=>'Iceland Krona',
    	'INR'=>'Indian Rupee',
    	'IDR'=>'Indonesian Rupiah',
    	'IQD'=>'Iraqi Dinar',
    	'ILS'=>'Israeli Shekel',
    	'JMD'=>'Jamaican Dollar',
    	'JPY'=>'Japanese Yen',
    	'JOD'=>'Jordanian Dinar',
    	'KZT'=>'Kazakhstan Tenge',
    	'KES'=>'Kenyan Shilling',
    	'KRW'=>'Korean Won',
    	'KWD'=>'Kuwaiti Dinar',
    	'LAK'=>'Lao Kip',
    	'LVL'=>'Latvian Lat',
    	'LBP'=>'Lebanese Pound',
    	'LSL'=>'Lesotho Loti',
    	'LRD'=>'Liberian Dollar',
    	'LYD'=>'Libyan Dinar',
    	'LTL'=>'Lithuanian Lita',
    	'MOP'=>'Macau Pataca',
    	'MKD'=>'Macedonian Denar',
    	'MGF'=>'Malagasy Franc',
    	'MWK'=>'Malawi Kwacha',
    	'MYR'=>'Malaysian Ringgit',
    	'MVR'=>'Maldives Rufiyaa',
    	'MTL'=>'Maltese Lira',
    	'MRO'=>'Mauritania Ougulya',
    	'MUR'=>'Mauritius Rupee',
    	'MXN'=>'Mexican Peso',
    	'MDL'=>'Moldovan Leu',
    	'MNT'=>'Mongolian Tugrik',
    	'MAD'=>'Moroccan Dirham',
    	'MZM'=>'Mozambique Metical',
    	'MMK'=>'Myanmar Kyat',
    	'NAD'=>'Namibian Dollar',
    	'NPR'=>'Nepalese Rupee',
    	'ANG'=>'Neth Antilles Guilder',
    	'NZD'=>'New Zealand Dollar',
    	'NIO'=>'Nicaragua Cordoba',
    	'NGN'=>'Nigerian Naira',
    	'KPW'=>'North Korean Won',
    	'NOK'=>'Norwegian Krone',
    	'OMR'=>'Omani Rial',
    	'XPF'=>'Pacific Franc',
    	'PKR'=>'Pakistani Rupee',
    	'XPD'=>'Palladium Ounces',
    	'PAB'=>'Panama Balboa',
    	'PGK'=>'Papua New Guinea Kina',
    	'PYG'=>'Paraguayan Guarani',
    	'PEN'=>'Peruvian Nuevo Sol',
    	'PHP'=>'Philippine Peso',
    	'XPT'=>'Platinum Ounces',
    	'PLN'=>'Polish Zloty',
    	'QAR'=>'Qatar Rial',
    	'ROL'=>'Romanian Leu',
    	'RUB'=>'Russian Rouble',
    	'WST'=>'Samoa Tala',
    	'STD'=>'Sao Tome Dobra',
    	'SAR'=>'Saudi Arabian Riyal',
    	'SCR'=>'Seychelles Rupee',
    	'SLL'=>'Sierra Leone Leone',
    	'XAG'=>'Silver Ounces',
    	'SGD'=>'Singapore Dollar',
    	'SKK'=>'Slovak Koruna',
    	'SIT'=>'Slovenian Tolar',
    	'SBD'=>'Solomon Islands Dollar',
    	'SOS'=>'Somali Shilling',
    	'ZAR'=>'South African Rand',
    	'LKR'=>'Sri Lanka Rupee',
    	'SHP'=>'St Helena Pound',
    	'SDD'=>'Sudanese Dinar',
    	'SRG'=>'Surinam Guilder',
    	'SZL'=>'Swaziland Lilageni',
    	'SEK'=>'Swedish Krona',
    	'TRY'=>'Turkey Lira',
    	'CHF'=>'Swiss Franc',
    	'SYP'=>'Syrian Pound',
    	'TWD'=>'Taiwan Dollar',
    	'TZS'=>'Tanzanian Shilling',
    	'THB'=>'Thai Baht',
    	'TOP'=>'Tonga Paanga',
    	'TTD'=>'Trinidad and Tobago Dollar',
    	'TND'=>'Tunisian Dinar',
    	'TRL'=>'Turkish Lira',
    	'USD'=>'U.S. Dollar',
    	'AED'=>'UAE Dirham',
    	'UGX'=>'Ugandan Shilling',
    	'UAH'=>'Ukraine Hryvnia',
    	'UYU'=>'Uruguayan New Peso',
    	'VUV'=>'Vanuatu Vatu',
    	'VEB'=>'Venezuelan Bolivar',
    	'VND'=>'Vietnam Dong',
    	'YER'=>'Yemen Riyal',
    	'YUM'=>'Yugoslav Dinar',
    	'ZMK'=>'Zambian Kwacha',
    	'ZWD'=>'Zimbabwe Dollar');
    }
    
    
    /**
     * @return Gpf_Db_CurrencyRate
     */
    private function saveDailyRate(Gpf_DateTime $date, $targetCurrncy, $rate) {
        Gpf_Log::debug('Saving rate for ' . $targetCurrncy . ' on ' . $date->toDate() . ', rate=' . $rate);
        $currency = new Gpf_Db_CurrencyRate();
        $currency->setValidFrom($date->toDate() . ' 00:00:00');
        $currency->setValidTo($date->toDate() . ' 23:59:59');
        $currency->setSourceCurrency('EUR');
        $currency->setTargetCurrency($targetCurrncy);
        $currency->setRate($rate);
        $currency->setType(Gpf_Db_CurrencyRate::TYPE_DAILY);
        $currency->save();
        return $currency;
    }

    public function loadHistoricalRates() {
        $rawXmlData = file_get_contents('http://www.ecb.europa.eu/stats/eurofxref/eurofxref-hist.xml');
        $xml = new SimpleXMLElement($rawXmlData);
        $lastDate = null;
        $lastRates = null;
        Gpf_Log::debug('Loading to db...');
        foreach ($xml->{"Cube"}->{"Cube"} as $dailyRates) {
            $attr = $dailyRates->attributes();
            $date = new Gpf_DateTime($attr['time']);
            Gpf_Log::debug('Loading rates for ' . $date->toDate());
            $dayBefore = new Gpf_DateTime($attr['time']);
            $dayBefore->addDay(1);
            if ($lastDate !== null && $dayBefore->toDate() != $lastDate->toDate()) {
                Gpf_Log::debug('Last date was ' . $lastDate->toDate() . '! Extending last saved rates validFrom parameters...');
                foreach($lastRates as $rate) {
                    Gpf_Log::debug('Extending validFrom for currency ' . $rate->getValidFrom() . ' - ' . $rate->getValidTo() . ' - ' . $rate->getTargetCurrency() . ' to value ' . $dayBefore->toDate() . ' 00:00:00');
                    $rate->setValidFrom($dayBefore->toDate() . ' 00:00:00');
                    $rate->update(array(Gpf_Db_Table_CurrencyRates::VALID_FROM));
                }
            }
            Gpf_Log::debug('Saving rates for ' . $date->toDate());
            $lastRates = array();
            foreach ($dailyRates->{"Cube"} as $currencyRate) {
                $info = $currencyRate->attributes();
                Gpf_Log::debug('Saving EUR to ' . $info['currency'] . ', rate=' . $info['rate']);
                $rate = $this->saveDailyRate($date, (String)$info['currency'], (Double)$info['rate']);
                $lastRates[] = $rate;
            }
            $lastDate = new Gpf_DateTime($attr['time']);
        }
        Gpf_Log::debug('Load complete');
    }


}
