<?php
/**
 *   @copyright Copyright (c) 2007 Quality Unit s.r.o.
 *   @author Andrej Harsani
 *   @package GwtPhpFramework
 *   @since Version 1.0.0
 *   $Id: Settings.class.php 18002 2008-05-13 18:31:39Z aharsani $
 *
 *   Licensed under the Quality Unit, s.r.o. Dual License Agreement,
 *   Version 1.0 (the "License"); you may not use this file except in compliance
 *   with the License. You may obtain a copy of the License at
 *   http://www.qualityunit.com/licenses/gpf
 *
 */

/**
 * @package GwtPhpFramework
 */
class Gpf_File_Config {
	/**
	 * @var Gpf_Io_File
	 */
    private $settingsFile;
    
    private $parameters = array();
    private $initialized = false;

    public function __construct($settingsFileName) {
        $this->settingsFile = new Gpf_Io_File($settingsFileName);
    }
    
    /**
     * @return array
     */
    public function getAll() {
        if(!$this->initialized) {
            $this->parameters = $this->readSettingsValues();
            $this->initialized = true;
        }
        return $this->parameters;
    }

    public function saveAll() {
        $this->writeSettingsValues();
    }

    public function hasSetting($name) {
        $this->getAll();
        return array_key_exists($name, $this->parameters);
    }

    public function forceReload($value = false) {
        $this->initialized = $value;
    }

    public function getSetting($name) {
        $this->getAll();
        if(array_key_exists($name, $this->parameters)) {
            return $this->parameters[$name];
        }

        throw new Gpf_Settings_UnknownSettingException($name);
    }

    public function getSettingWithDefaultValue($name, $defaultValue) {
        // obsolete
        // to be deleted
        $this->getAll();
        if(array_key_exists($name, $this->parameters)) {
            return $this->parameters[$name];
        }

        return $defaultValue;
    }

    public function setSetting($name, $value, $flush = true) {
        $this->getAll();
        if(array_key_exists($name, $this->parameters) && $this->parameters[$name] == $value) {
            return;
        }
        $this->parameters[$name] = $value;
        if($flush) {
            $this->writeSettingsValues();
        }
    }

    public function getFileName(){
    	return $this->getSettingFileName();
    }
    
    public function getSettingFileName() {
        return $this->settingsFile->getFileName();
    }

    public function isExists() {
        return $this->settingsFile->isExists();
    }

    public function removeSetting($settingName, $flush = true) {
        if (!$this->hasSetting($settingName)) {
            return;
        }
        unset($this->parameters[$settingName]);
        if ($flush) {
            $this->writeSettingsValues();
        }
    }

    protected function isFileContentOk($loadedArray) {
        return true;
    }

    private function readSettingsValues() {
        if(!$this->settingsFile->isExists()) {
            return array();
        }
        $this->settingsFile->open();

        $values = array();
        $lines = $this->readFileAsArray();

        foreach($lines as $line) {
            if(false !== strpos($line, '<?') || false !== strpos($line, '?>')) {
                continue;
            }
            $pos = strpos($line, '=');
            if($pos === false) {
                continue;
            }
            $name = substr($line, 0, $pos);
            $value = substr($line, $pos + 1);
            $values[$name] = rtrim($value);
        }
        return $values;
    }

    private function readFileAsArray() {
        for ($i = 1; $i <= 5; $i++) {
            $lines = $this->settingsFile->readAsArray();
            if ($this->isFileContentOk($lines)) {
                return $lines;
            }
            usleep(round(rand(0, 100)*1000));
        }

        throw new Gpf_Exception('Could not read settings file: ' . ' ' . $this->getFileName());
    }

    protected function isSettingsFileOk(Gpf_Io_File $file) {
        try {
            return ($file->getSize() > 0) || ($this->getFileDataLength($file) > 0);
        } catch (Exception $e) {
            return false;
        }
    }

    private function getFileDataLength(Gpf_Io_File $file) {
        $data = file_get_contents($file->getFileName());
        return strlen($data);
    }

    private function writeSettingsValues() {
        $settingsTmpFile = new Gpf_Io_File($this->getFileName() . '_' . date("Y-m-d_H.i.s") . '_'. uniqid() .'.tmp');

        $this->writeSettingToFile($settingsTmpFile);

        if (!$this->isSettingsFileOk($settingsTmpFile)) {
            $settingsTmpFile->delete();
            throw new Gpf_Exception('Unable to save settings file! ' . date('Y-m-d H:i:s', time()));
        }

        for ($i = 1; $i <= 5; $i++) {
            try {
                $this->copyFile($settingsTmpFile, $this->settingsFile, 0777);
                if (!$this->isSettingsFileOk($this->settingsFile)) {
                    throw new Gpf_Exception('');
                }
            } catch (Exception $e) {
                $this->writeSettingToFile($this->settingsFile);
            }
            if ($this->isSettingsFileOk($this->settingsFile)) {
                $settingsTmpFile->delete();
                return;
            }
            usleep(round(rand(0, 100)*1000));
        }
        throw new Gpf_Exception('Unable to save settings file! (Temp file is OK: '. $settingsTmpFile->getFileName().')');
    }

    private function writeSettingToFile(Gpf_Io_File $file) {
        if (defined('ENABLE_ENGINECONFIG_LOG')) {
            Gpf_Log::info('(writeSettingsValues - before write) file ' . @$file->getFileName() . ' size: ' . @$file->getSize() . ', permissions: ' . @$file->getFilePermissions() . ', owner: ' . @$file->getFileOwner());
        }
        $file->open('w');

        $text = '<?php /*' . "\n";
        foreach($this->parameters as $key => $value) {
            $text .= $key . '=' . $value . "\r\n";
        }
        $text .= '*/ ?>';
        $file->write($text);
        $file->close();

        if (defined('ENABLE_ENGINECONFIG_LOG')) {
            Gpf_Log::info('(writeSettingsValues - after write) file ' . @$file->getFileName() . ' size: ' . @$file->getSize() . ', permissions: ' . @$file->getFilePermissions() . ', owner: ' . @$file->getFileOwner());
        }
    }

    protected function copyFile(Gpf_Io_File $source, Gpf_Io_File $target, $mode = null) {
        $target->open('w');
        $target->write($source->getContents());
        if($mode !== null) {
            @chmod($target->getFileName(), $mode);
        }
    }
}

?>
