<?php
/**
 *   @copyright Copyright (c) 2012 Quality Unit s.r.o.
 *   @package PostAffiliatePro
 *   @author Juraj Simon
 *
 *   Licensed under the Quality Unit, s.r.o. Standard End User License Agreement,
 *   Version 1.0 (the "License"); you may not use this file except in compliance
 *   with the License. You may obtain a copy of the License at
 *   http://www.qualityunit.com/licenses/license
 */

abstract class Gpf_RestApi_Api extends Gpf_ModuleBase {

    const KEY_FIELD_NAME = 'apikey';
    const API_LOG_GOUP = 'api_';
    const API_LOG_TYPE = 'A';

    private $callId = null;
    private $encoders = array();

    /**
     * @var Gpf_RestApi_Response_Encoder
     */
    private $activeEncoder;

    protected abstract function getHndlerClassPrefix();
    
    protected abstract function getHndlerClassSuffix();
    
    protected function initEncoders() {
        $this->encoders[] = new Gpf_RestApi_Response_Encoder_Json();
        $this->encoders[] = new Gpf_RestApi_Response_Encoder_Xml();
        $this->encoders[] = new Gpf_RestApi_Response_Encoder_Yaml();
    }

    /**
     * @return Gpf_RestApi_Response_Encoder
     */
    private function getEncoder($encoderType) {
        foreach ($this->encoders as $encoder) {
            if (strtolower($encoder->getType() == strtolower($encoderType))) {
                return $encoder;
            }
        }
        return null;
    }

    /**
     * @return Gpf_RestApi_Response_Encoder
     * @throws Gpf_Exception
     */
    private function getEncoderAndUpdateHandlerString() {
        $handler = $this->getHandler();
        $pos = strrpos($handler, '.');
        if ($pos === false) {
            return new Gpf_RestApi_Response_Encoder_Json();
        }
        $encoderType = substr($handler, $pos + 1);
        $encoder = $this->getEncoder($encoderType);
        if ($encoder == null) {
            return new Gpf_RestApi_Response_Encoder_Json();
        }
        $this->setHandler(substr($handler, 0, $pos));
        return $encoder;
    }

    protected function initLogger() {
        $this->callId = self::API_LOG_GOUP . Gpf_Common_String::generateId(12);
        $logger = Gpf_Log_Logger::getInstance();
        $logger->setGroup($this->callId);
        $logger->add(Gpf_Log_LoggerDatabase::TYPE, Gpf_Log::ERROR);
        $logger->setType(self::API_LOG_TYPE);
    }

    protected function authenticate($key) {
        Gpf_Log::debug('Authenticating for key *****');
        $auth = new Gpf_RestApi_Auth_Key();
        $auth->authenticate($key);
        Gpf_Log::debug('Auth complete');
        $this->roleType = Gpf_Session::getAuthUser()->getRoleTypeId();
        Gpf_Session::refreshAuthUser();
    }

    public function startAndGet() {
        $this->execute();
    }
    
    protected function initSession() {
        $sessionId = null;
        if (array_key_exists('S', $_REQUEST) && $_REQUEST['S'] != '') {
            $sessionId = $_REQUEST['S'];
        }
        Gpf_Session::create($this, $sessionId);
        //$this->checkApplication();
    }

    protected function onStart() {
        parent::onStart();
    }

    private function execute() {
        Gpf_Log::debug('API call from IP:' . Gpf_Http::getRemoteIp() . ', User agent: ' . $this->getUserAgent() . ', Referer: ' . $this->getReferer());

        try {
            $this->activeEncoder = $this->getEncoderAndUpdateHandlerString();
        } catch (Gpf_Exception $e) {
            $messgae = $this->_('Can not determine proper encoder from handler string');
            $this->insertHeaderByCode(500, $messgae);
            echo $messgae;
            exit();
        }

        $key = $this->getKey();
        if ($key == null) {
            $this->returnError(400, $this->_('Key not found in request!'));
        }

        try {
            $this->authenticate($key);
        } catch (Gpf_Exception $e) {
            $this->returnError(500, $this->_('Auth failed.'));
        }
        
        $versionParser = new Gpf_RestApi_VersionParser();
        $version = $versionParser->getVersion($this->getHndlerClassPrefix(), $this->getHndlerClassSuffix());
        $handlerClass = $this->getHndlerClassPrefix() . $version . $this->getHndlerClassSuffix();
        try {
            $handler = new $handlerClass;
        } catch (Gpf_ClassNotDefined $e) {
            $this->returnError(404, $this->_('Api version %s not found', $version));
        }
        
        $data = new Gpf_RestApi_Data();
        try {
            $this->fillData($data);
            $data->setRequestMethod(@$_SERVER['REQUEST_METHOD']);
            $handler->setData($data);
            $this->returnResponse($handler->handle());
        } catch (Gpf_RestApi_BadHandlerException $e) {
            $this->returnError(400, $this->_('Handler is missing or not valid'));
        } catch (Gpf_RestApi_Param_WrongFormatException $e) {
            $this->returnError(400, $this->_('One or more parameters in handler are wrong'));
        } catch (Gpf_RestApi_ProcessingException $e) {
            $this->returnErrorFromException($e);
        } catch (Gpf_Exception $e) {
            Gpf_Log::debug('Error during API execution: ' . $e->getMessage());
            $this->returnError(500, $this->_('Error during API execution' . $e->getMessage()));
        }
        $this->returnError(404, $this->_('No handler found'));
    }
    
    private function fillData(Gpf_RestApi_Data $data) {
        $data->fillFromRequestVar($_REQUEST);
        if (strtolower(trim(@$_SERVER['REQUEST_METHOD'])) == 'put' || strtolower(trim(@$_SERVER['REQUEST_METHOD'])) == 'delete') {
            $putFields = array();
            parse_str(file_get_contents('php://input'), $putFields);
            foreach ($putFields as $name => $value) {
                $data->setParam($name, $value);
            }
        }
    }
    
    private function returnErrorFromException(Gpf_RestApi_ProcessingException $e) {
        Gpf_Log::debug('Error during API execution: ' . $e->getMessage());
        $this->returnError($e->getCode(), $e->getShortMessage());
    }
    
    private function returnResponse($response) {
        if ($response !== null && is_object($response)) {
            $this->insertHeaderByCode(200);
            $this->activeEncoder->encodeAndEcho($response);
            exit();
        }
    }
    
    protected function returnError($code, $message) {
        Gpf_Log::debug($code . ' - ' . $message);
        $this->insertHeaderByCode($code, $message);
        $this->activeEncoder->encodeAndEcho(new Gpf_RestApi_Response_Error(new Gpf_RestApi_ProcessingException($code, $message)));
        exit();
    }
    
    private function insertHeaderByCode($code, $message = '') {
        switch ($code) {
            case 200:
                $this->setHeader('HTTP/1.1 200 OK', $code, $message);
                break;
            case 400:
                $this->setHeader('HTTP/1.1 400 Bad Request', $code, $message);
                break;
            case 401:
                $this->setHeader('HTTP/1.1 401 Unauthorized', $code, $message);
                break;
            case 403:
                $this->setHeader('HTTP/1.1 403 Forbidden', $code, $message);
                break;
            case 404:
                $this->setHeader('HTTP/1.1 404 Not Found', $code, $message);
                break;
            case 503:
                $this->setHeader('HTTP/1.1 503 Service Unavailable', $code, $message);
                break;
            default:
                $this->setHeader('HTTP/1.1 500 Internal Server Error', $code, $message);
        }
    }
    
    private function setHandler($handlerString) {
        if (array_key_exists('handler', $_REQUEST)) {
            $_REQUEST['handler'] = $handlerString;
            return;
        }
        throw new Gpf_Exception($this->_('No handler defined'));
    }
    
    protected function getHandler() {
        if (array_key_exists('handler', $_REQUEST)) {
            return $_REQUEST['handler'];
        }
        throw new Gpf_Exception($this->_('No handler defined'));
    }
    
    private function setHeader($header, $code = null, $warning = null) {
        header($header);
        Gpf_Http::setHeader('PapApiReqId', $this->callId);
        if ($code != null && $warning != null) {
            Gpf_Http::setHeader('Warning', sprintf('%s "%s"', $code, $warning));
            Gpf_Log::debug($warning);
        }
    }
    
    private function getReferer() {
        if (array_key_exists('HTTP_REFERER', $_SERVER)) {
            return $_SERVER['HTTP_REFERER'];
        }
        return '';
    }
    
    private function getUserAgent() {
        if (array_key_exists('HTTP_USER_AGENT', $_SERVER)) {
            return $_SERVER['HTTP_USER_AGENT'];
        }
        return '';
    }
    
    private function getKey() {
        return @$_REQUEST[self::KEY_FIELD_NAME];
    }
}
?>
