<?php
/**
 *   @copyright Copyright (c) 2012 Quality Unit s.r.o.
 *   @package GwtPhpFramework
 *   @author Michal Bebjak
 *
 *   Licensed under the Quality Unit, s.r.o. Dual License Agreement,
 *   Version 1.0 (the "License"); you may not use this file except in compliance
 *   with the License. You may obtain a copy of the License at
 *   http://www.qualityunit.com/licenses/gpf
 */

/**
 * Client for basic RestApi calls
 * !!! This class can not use any other Gpf class as it can be used also in non Gpf projects !!!
 */
class Gpf_RestApi_Client {
    
	private $apiUrl, $apiKey;
	private $timeout = 20;
	
    public function __construct($apiUrl, $apiKey) {
        $this->apiUrl = $apiUrl;
        $this->apiKey = $apiKey;
    }
    
    public function get($handlerUrl, array $params = array()) {
    	return $this->sendRequest($handlerUrl, Gpf_RestApi_Client_Request::METHOD_GET, $params);
    }
    
    public function post($handlerUrl, array $params = array()) {
    	return $this->sendRequest($handlerUrl, Gpf_RestApi_Client_Request::METHOD_POST, $params);
    }
    
    public function put($handlerUrl, array $params = array()) {
    	return $this->sendRequest($handlerUrl, Gpf_RestApi_Client_Request::METHOD_PUT, $params);
    }
    
    public function delete($handlerUrl, array $params = array()) {
    	return $this->sendRequest($handlerUrl, Gpf_RestApi_Client_Request::METHOD_DELETE, $params);
    }
    
    private function sendRequest($handlerUrl, $method, array $params = array()) {
        Gpf_Log::debug('sending request of type ' . $method);
        Gpf_Log::debug('sending request to url ' . $handlerUrl);
        Gpf_Log::debug('params ' . var_export($params, true));
        Gpf_Log::debug('api url:' . $this->apiUrl);
    	$request = new Gpf_RestApi_Client_Request($this->apiUrl, $handlerUrl, $this->apiKey, $method, $params);
    	return $request->send();
    }
}

class Gpf_RestApi_Client_Request {
	
	const METHOD_GET = "GET";
	const METHOD_POST = "POST";
	const METHOD_PUT = "PUT";
	const METHOD_DELETE = "DELETE";
	
	
	private $apiUrl, $handlerUrl, $apiKey, $method, $params, $timeout = 20;
	
	public function __construct($apiUrl, $handlerUrl, $apiKey, $method, $params = array()) {
		$this->setApiUrl($apiUrl);
		$this->handlerUrl = ltrim($handlerUrl, '/');
		$this->apiKey = $apiKey;
		$this->method = $method;
		$this->params = $params;
	}
	
	/**
	 * @return stdObject
	 * @throws Exception if curl failed
	 * @throws Gpf_RestApi_Client_ResponseException if api responded with exception
	 */
	public function send() {
		$service_url = $this->apiUrl.'?handler='.urlencode($this->handlerUrl).'&apikey='.$this->apiKey;
		if ($this->method == self::METHOD_GET) {
			foreach ($this->params as $name => $value) {
				$service_url .= "&".urlencode($name).'='.urlencode($value);
			}
		}
		
		Gpf_Log::debug('curl url:' . $service_url);
		Gpf_Log::debug('curl params: ', var_export($this->params, true));
		$curl = curl_init($service_url);
		curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
		curl_setopt($curl, CURLOPT_CONNECTTIMEOUT, $this->timeout);
		curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);
		curl_setopt($curl, CURLOPT_DNS_USE_GLOBAL_CACHE, false);
		curl_setopt($curl, CURLOPT_DNS_CACHE_TIMEOUT, 1);
		curl_setopt($curl, CURLOPT_HTTPHEADER, array('Expect:'));
		if ($this->method != self::METHOD_GET) {
			curl_setopt($curl, CURLOPT_CUSTOMREQUEST, $this->method);
			curl_setopt($curl, CURLOPT_POSTFIELDS, http_build_query($this->params));
		}
		
		$curl_response = curl_exec($curl);
		Gpf_Log::debug('response: ' . var_export($curl_response, true));
		if ($curl_response === false) {
			$info = curl_getinfo($curl);
			$err = curl_error($curl);
			curl_close($curl);
			throw new Exception('Error occured during curl exec. Additioanl info: ' . var_export($info) . 'err: ' . $err);
		}
		curl_close($curl);
		$decoded = json_decode($curl_response);
		Gpf_Log::debug('decoded response' . var_export($decoded, true));
		if (isset($decoded->response->status) && $decoded->response->status == 'ERROR') {
			throw new Gpf_RestApi_Client_ResponseException($decoded->response->errormessage, $decoded->response->statuscode);
		}
		if (!isset($decoded->response)) {
		    throw new Gpf_RestApi_Client_ResponseException($curl_response);
		}
		return $decoded->response;
	}
	
	public function setApiUrl($apiUrl) {
		$this->apiUrl = rtrim(str_replace("index.php", "", $apiUrl), '/') . '/index.php';
	}
}

class Gpf_RestApi_Client_ResponseException extends Exception {
	public function __construct($message = "", $code = 0) {
		parent::__construct($message, $code);
	}
}

?>
