<?php
/**
 *   @copyright Copyright (c) 2007 Quality Unit s.r.o.
 *   @author Matej Kendera
 *   @package PostAffiliatePro
 *   @since Version 1.0.0
 *
 *   Licensed under the Quality Unit, s.r.o. Standard End User License Agreement,
 *   Version 1.0 (the "License"); you may not use this file except in compliance
 *   with the License. You may obtain a copy of the License at
 *   http://www.postaffiliatepro.com/licenses/license
 *
 */

/**
 * @package PostAffiliatePro
 */
class Pap_Features_NetworkAccountPrepaid_Main extends Gpf_Plugins_Handler {

    const LOW_NOTIFICATION_SENT = 'NetworkPrepaid_LowNotificationSent';
    const IS_ACCOUNT_PREPAID = 'NetworkPrepaid_IsAccountPrepaid';

    /**
     * @return Pap_Features_NetworkAccountPrepaid_Main
     */
    public static function getHandlerInstance() {
        return new Pap_Features_NetworkAccountPrepaid_Main();
    }

    public function initSettings($context) {
        $context->addDbSetting(Pap_Features_NetworkAccountPrepaid_Config::LOW_CREDIT_LIMI, 50);
        $context->addDbSetting(Pap_Features_NetworkAccountPrepaid_Config::SUSPEND_CREDIT_LIMIT, 0);
        $context->addDbSetting(Pap_Features_NetworkAccountPrepaid_Config::LOW_CREDIT_NOTIFICATION_ON_EVERY_SALE, Gpf::NO);
        $context->addDbSetting(self::LOW_NOTIFICATION_SENT, Gpf::NO, true);
        $context->addDbSetting(self::IS_ACCOUNT_PREPAID, Gpf::YES, true);
    }

    public function getNextInvoiceDateFrom(Gpf_DateTime $accountDateInserted) {
        $accountDateInserted->addDay(-1);
    }

    public function doAfterInvoicePaid(Gpf_DbEngine_RowBase $invoice) {
        $accountId = $invoice->getAccountId();

        $accountSettings = Gpf_Settings::getAccountSettings($accountId);
        $accountSettings->set(self::LOW_NOTIFICATION_SENT, Gpf::NO);

        if (!$this->isAccountPrepaid($accountId)) {
            return;
        }

        $accountBallance = $this->getAccountBallacne($accountId);

        if ($accountBallance >= Gpf_Settings::get(Pap_Features_NetworkAccountPrepaid_Config::SUSPEND_CREDIT_LIMIT)) {
            $this->unsuspendAccount($accountId);
        }
    }

    public function doAfterTransactionSave(Pap_Common_Transaction $transaction) {
        $accountId = $transaction->getAccountId();

        if ($accountId == Gpf_Db_Account::DEFAULT_ACCOUNT_ID) {
            Gpf_Log::info('NetworkAccountPrepaidFeature - ending network owner account');
            return;
        }

        if (!$this->isAccountPrepaid($accountId)) {
            Gpf_Log::info('NetworkAccountPrepaidFeature - ending network account is not prepaid: ' . $accountId);
            return;
        }

        $accountBallance = $this->getAccountBallacne($accountId);

        Gpf_Log::info('NetworkAccountPrepaidFeature ballance after transactionid ' . $transaction->getId() . ' is ' . $accountBallance);

        if ($accountBallance < Gpf_Settings::get(Pap_Features_NetworkAccountPrepaid_Config::SUSPEND_CREDIT_LIMIT)) {
            $this->suspendAccount($accountId);
            Gpf_Log::info('NetworkAccountPrepaidFeature ballance is lower than suspend limit, account ' . $accountId .' suspended.');
            return;
        }

        if ($accountBallance < Gpf_Settings::get(Pap_Features_NetworkAccountPrepaid_Config::LOW_CREDIT_LIMI)) {
            $this->processLowCreditEmail($accountId);
            Gpf_Log::info('NetworkAccountPrepaidFeature ballance is lower than low credit limit, account: ' . $accountId);
        }
    }

    public function doLoadAccountsForm(Gpf_Rpc_Form $accountsForm) {
        $accountSettings = Gpf_Settings::getAccountSettings($accountsForm->getFieldValue('Id'));
        $accountsForm->addField('prepaidAccount', $accountSettings->get(self::IS_ACCOUNT_PREPAID));
    }

    // @service account_prepaid_setting write
    
    public function doSaveAccountsForm(Gpf_Rpc_Form $accountsForm) {
        $accountId = $accountsForm->getFieldValue('Id');
        if ($accountId == '') {
            return;
        }
        try {
            $prepaidAccountValue = $accountsForm->getFieldValue('prepaidAccount');
        } catch (Gpf_Exception $e) {
            return;
        }

        $accountSettings = Gpf_Settings::getAccountSettings($accountId);

        $isPrepaidSettingChanged = false;
        if ($accountSettings->get(self::IS_ACCOUNT_PREPAID) != $prepaidAccountValue) {
            $isPrepaidSettingChanged = true;
        }

        if (!$this->hasEditPrepaidSettingPrivilege() && $isPrepaidSettingChanged) {
            $accountsForm->setErrorMessage($this->_('You does not have privilege to edit prepaid settings!'));
            return;
        }
        $accountSettings->set(self::IS_ACCOUNT_PREPAID, $prepaidAccountValue);
    }

    private function processLowCreditEmail($accountId) {
        $accountSettings = Gpf_Settings::getAccountSettings($accountId);
        if (Gpf_Settings::get(Pap_Features_NetworkAccountPrepaid_Config::LOW_CREDIT_NOTIFICATION_ON_EVERY_SALE) == Gpf::YES) {
            $this->sendLowCreditEmail($accountId);
        } else {
            if ($accountSettings->get(self::LOW_NOTIFICATION_SENT) != Gpf::YES) {
                $this->sendLowCreditEmail($accountId);
            }
        }

        $accountSettings->set(self::LOW_NOTIFICATION_SENT, Gpf::YES);
    }

    private function sendLowCreditEmail($accountId) {
        $account = Pap_Account::getAccount($accountId);
        $mail = new Pap_Features_NetworkAccountPrepaid_MailAccountLowCreditMerchant();
        $mail->setAccount($account);
        $mail->addRecipient($account->getEmail());
        $mail->send();
    }

    private function suspendAccount($accountId) {
        $account = Pap_Account::getAccount($accountId);
        $account->setStatus(Gpf_Db_Account::SUSPENDED);
        $account->save();
    }

    private function unsuspendAccount($accountId) {
        $account = Pap_Account::getAccount($accountId);
        $account->setStatus(Gpf_Db_Account::APPROVED);
        $account->save();
    }

    private function getAccountBallacne($accountId) {
        $statsData = new Pap_Features_AffiliateNetwork_AccountingStatsData();
        $params = new Gpf_Rpc_Params();
        $params->add('filters', array(array("accountid", "E", $accountId)));
        return $statsData->load($params)->getValue('ballance');
    }

    private function isAccountPrepaid($accountId) {
        $accountSettings = Gpf_Settings::getAccountSettings($accountId);
        return $accountSettings->get(self::IS_ACCOUNT_PREPAID) == Gpf::YES;
    }

    private function hasEditPrepaidSettingPrivilege() {
        return Gpf_Session::getAuthUser()->hasPrivilege(Pap_Privileges::ACCOUNT_PREPAID_SETTING, Pap_Privileges::P_WRITE);
    }
}
?>