<?php
/**
 *   @copyright Copyright (c) 2007 Quality Unit s.r.o.
 *   @author Michal Bebjak
 *   @package PostAffiliatePro
 *   @since Version 1.0.0
 *
 *   Licensed under the Quality Unit, s.r.o. Standard End User License Agreement,
 *   Version 1.0 (the "License"); you may not use this file except in compliance
 *   with the License. You may obtain a copy of the License at
 *   http://www.postaffiliatepro.com/licenses/license
 *
 */

/**
 * @package PostAffiliatePro
 */
class Pap_Stats_Transactions extends Pap_Stats_Base {

    /**
     * @var Pap_Stats_Refunds
     */
    private $refunds;

    /**
     * @var Pap_Stats_Chargebacks
     */
    private $chargebacks;

    /**
     * @var Pap_Stats_Computer_Transactions
     */
    private $computer = null;
    private $transactionType = null;
    protected $commissionTypeId = null;
    protected $tier = null;
    private $name;

    public function __construct(Pap_Stats_Params $params) {
        parent::__construct($params);
        $this->name = $this->_('All');
    }

    private function init() {
        if ($this->computer != null) {
            return;
        }
        $this->computer = $this->createComputer();
        $this->computer->setTransactionType($this->transactionType);
        $this->computer->setCommissionTypeId($this->commissionTypeId);
        $this->computer->setTier($this->tier);
        $this->computer->computeStats();
    }

    /**
     * @return Pap_Stats_Computer_Transactions
     */
    protected function createComputer() {
        if ($this->params->isDateApprovedFromDefined() || $this->params->isDateApprovedToDefined() || $this->params->isDestinationURLDefined()) {
            return new Pap_Stats_Computer_Transactions($this->params);
        }
        if ($this->params->isAffiliateIdDefined()) {
            return new Pap_Stats_Computer_TransactionsGroupedAff($this->params);
        }
        return new Pap_Stats_Computer_TransactionsGrouped($this->params);
    }

    public function setTransactionType($type) {
        $this->transactionType = $type;
        $this->name = Pap_Common_Constants::getTypeAsText($type);
    }

    public function setCommissionTypeId($commTypeId) {
        $this->commissionTypeId = $commTypeId;
        if ($commTypeId != null) {
            $this->loadNameFromCommissionType($commTypeId);
        }
    }

    public function setTier($tier) {
        $this->tier = $tier;
    }

    protected function getValueNames() {
        return array('count', 'commission', 'totalCost', 'refunds', 'chargebacks', 'name', 'type', 'commissiontypeid', 'campaignName');
    }

    private static $commissionTypesCache = array();

    /**
     * @return Pap_Db_CommissionType
     */
    private function loadCommissionType($commTypeId) {
        if (!array_key_exists($commTypeId, self::$commissionTypesCache)) {
            $commissionType = new Pap_Db_CommissionType();
            $commissionType->setId($commTypeId);
            try {
                $commissionType->load();
            } catch (Gpf_Exception $e) {
                $commissionType->setName($this->_('Unknown'));
            }
            self::$commissionTypesCache[$commTypeId] = $commissionType;
        }
        return self::$commissionTypesCache[$commTypeId];
    }

    private function loadNameFromCommissionType($commTypeId) {
        $commissionType = $this->loadCommissionType($commTypeId);
        $this->name = $commissionType->getName();
    }

    public function getName() {
        return $this->name;
    }

    private static $campaignCache = array();

    public function getCampaignName() {
        if ($this->commissionTypeId == null) {
            return '';
        }
        if (!array_key_exists($this->commissionTypeId, self::$campaignCache)) {
            $commissionType = $this->loadCommissionType($this->commissionTypeId);
            self::$campaignCache[$this->commissionTypeId] = Pap_Common_Campaign::getCampaignById($commissionType->getCampaignId());
        }
        return $this->_localize(self::$campaignCache[$this->commissionTypeId]->getName());
    }

    public function getType() {
        return $this->transactionType;
    }

    /**
     * @return Pap_Stats_Data_Commission
     */
    public function getCommission() {
        $this->init();
        return $this->computer->getCommissions();
    }

    /**
     * @return Pap_Stats_Data_Commission
     */
    public function getTotalCost() {
        $this->init();
        return $this->computer->getTotalCost();
    }

    /**
     * @return Pap_Stats_Data_Commission
     */
    public function getCount() {
        $this->init();
        return $this->computer->getCount();
    }

    /**
     * @return Pap_Stats_Refunds
     */
    public function getRefunds() {
        if ($this->refunds == null) {
            $this->refunds = $this->initRefundChargeBackClass(new Pap_Stats_Refunds($this->params));
        }
        return $this->refunds;
    }

    /**
     * @return Pap_Stats_Chargebacks
     */
    public function getChargebacks() {
        if ($this->chargebacks == null) {
            $this->chargebacks = $this->initRefundChargeBackClass(new Pap_Stats_Chargebacks($this->params));
        }
        return $this->chargebacks;
    }

    private function initRefundChargeBackClass(Pap_Stats_RefundChargeback $refundchargeback) {
        $refundchargeback->setTransactionType($this->transactionType);
        $refundchargeback->setCommissionTypeId($this->commissionTypeId);
        $refundchargeback->setTier($this->tier);
        return $refundchargeback;
    }

    public function getCommissionTypeId() {
        return $this->commissionTypeId;
    }
}
?>
