<?php
/**
 *   @copyright Copyright (c) 2007 Quality Unit s.r.o.
 *   @author Michal Bebjak
 *   @package PostAffiliatePro
 *   @since Version 1.0.0
 *
 *   Licensed under the Quality Unit, s.r.o. Standard End User License Agreement,
 *   Version 1.0 (the "License"); you may not use this file except in compliance
 *   with the License. You may obtain a copy of the License at
 *   http://www.postaffiliatepro.com/licenses/license
 *
 */

/**
 * @package PostAffiliatePro
 */
class AutoApprovalCommissions_Runner extends Gpf_Tasks_LongTask {

    private $accountNottificationEmails = array();

    /**
     * @var Gpf_Log_Logger
     */
    private $logger;

    public function getName() {
        return $this->_('Auto approval commissions');
    }
    
    protected function getTransactionsListSelect() {
        $types = array(Pap_Common_Constants::TYPE_SALE, Pap_Common_Constants::TYPE_ACTION, Pap_Common_Constants::TYPE_LEAD, Pap_Common_Constants::TYPE_RECURRING);
        
        $select = new Gpf_SqlBuilder_SelectBuilder();
         
        $select->select->addAll(Pap_Db_Table_Transactions::getInstance(), 't');
        $select->from->add(Pap_Db_Table_Transactions::getName(), 't');
        $select->from->addInnerJoin(Pap_Db_Table_CommissionTypeAttributes::getName(), 'cta', 
            'cta.'.Pap_Db_Table_CommissionTypeAttributes::COMMISSION_TYPE_ID.'=t.'.Pap_Db_Table_Transactions::COMMISSIONTYPEID. ' 
            and (cta.'.Pap_Db_Table_CommissionTypeAttributes::NAME."='".AutoApprovalCommissions_Main::AUTO_APPROVAL_COMMISSIONS_DAYS."' 
            and ".Pap_Db_Table_CommissionTypeAttributes::VALUE . '<>0)
            and (unix_timestamp(t.dateinserted) + (cta.'.Pap_Db_Table_CommissionTypeAttributes::VALUE.' * 86400)) <= unix_timestamp()
            ');

        $select->where->add(Pap_Db_Table_Transactions::R_TYPE, "IN", $types);
        $select->where->add(Pap_Db_Table_Transactions::R_STATUS, "=", Pap_Common_Constants::STATUS_PENDING);
        return $select;
    }

    protected function execute() {
        $this->log('Runner started');
        $commTypeAttr = Pap_Db_Table_CommissionTypeAttributes::getInstance();
        $i = 0;
        foreach ($this->getTransactionsListSelect()->getAllRowsIterator() as $transactionRecord) {
            $transaction = new Pap_Common_Transaction();
            $transaction->fillFromRecord($transactionRecord);
            $transaction->setStatus(Pap_Common_Constants::STATUS_APPROVED);
            $note = $commTypeAttr->getCommissionTypeAttribute($transaction->getCommissionTypeId(), AutoApprovalCommissions_Main::AUTO_APPROVAL_COMMISSIONS_NOTE)->getValue();
            if ($note != '') {
                $transaction->setMerchantNote($note);
            }
            $transaction->update(array(Pap_Db_Table_Transactions::R_STATUS, Pap_Db_Table_Transactions::MERCHANTNOTE, Pap_Db_Table_Transactions::DATE_APPROVED));
            $this->setLogGroup($transaction->getLogGroupId());
            $this->log('Transacton id: ' . $transaction->getId() . ' has been approved.');
            if ($this->isMerchantNotificationsEnabled()) {
                $this->sendMerchatnNotification($transaction);
            }
            $this->approveRefund($transaction->getId());
            $this->approveRecurringCommission($transaction->getId());
            $i++;
            if($this->isTimeToInterrupt()) {
                $this->log('approved transactions count: ' . $i);
                $this->interrupt(0);
            }
        }
        $this->log('approved transactions count: ' . $i);
        $this->log('Runner ended');
    }

    private function approveRefund($transactionId) {
        $transaction = new Pap_Common_Transaction();
        $transaction->setId($transactionId);
        $refundTransaction = $transaction->getRefundOrChargebackTransaction();
        if (is_null($refundTransaction)) {
            $this->log('Transacton id: ' . $transactionId . ' does not have refund transaction.' );
            return;
        }
        if ($refundTransaction->getStatus() == Pap_Common_Constants::STATUS_PENDING) {
            $refundTransaction->setStatus(Pap_Common_Constants::STATUS_APPROVED);
            if ($refundTransaction->getDateApproved() == null) {
                $refundTransaction->setDateApproved(Gpf_Common_DateUtils::now());
            }
            $refundTransaction->update(array(Pap_Db_Table_Transactions::R_STATUS, Pap_Db_Table_Transactions::DATE_APPROVED));
            $this->log('Refund commission (' . $refundTransaction->getId() . ') for transacton id: ' . $transactionId . ' is approved.' );
        }
    }

    private function approveRecurringCommission($transactionId) {
        $recurringCommission = new Pap_Db_RecurringCommission();
        $recurringCommission->setTransactionId($transactionId);
        try {
            $recurringCommission->loadFromData(array(Pap_Db_Table_RecurringCommissions::TRANSACTION_ID));
            if ($recurringCommission->getStatus() == Pap_Common_Constants::STATUS_PENDING) {
                $recurringCommission->setStatus(Pap_Common_Constants::STATUS_APPROVED);
                $recurringCommission->update(array(Pap_Db_Table_RecurringCommissions::STATUS));
                $this->log('Recurring commission for transacton id: ' . $transactionId . ' is approved.' );
            }
        } catch (Gpf_DbEngine_NoRowException $e) {
            $this->log('Transacton id: ' . $transactionId . ' does not have recurring commission.' );
        }
    }

    /**
     * @return boolean
     */
    private function isMerchantNotificationsEnabled() {
        return Gpf_Settings::get(AutoApprovalCommissions_Config::ENABLE_MERCHANT_NOTIFICATIONS) == Gpf::YES;
    }

    private function sendMerchatnNotification(Pap_Common_Transaction $transaction) {
        $user = Pap_Common_User::getUserById($transaction->getUserId());

        $merchantEmail = $this->getMerchantNotificationEmail($transaction->getAccountId());
        $mailTemplate = new AutoApprovalCommissions_MerchantOnSaleApproved();
        $mailTemplate->setUser($user);
        $mailTemplate->setTransaction($transaction);
        $mailTemplate->addRecipient($merchantEmail);
        $mailTemplate->send();
    }

    private function getMerchantNotificationEmail($accountId) {
        if (!array_key_exists($accountId, $this->accountNottificationEmails)) {
            $accountSettings = Gpf_Settings::getAccountSettings($accountId);
            $this->accountNottificationEmails[$accountId] = $accountSettings->get(Pap_Settings::MERCHANT_NOTIFICATION_EMAIL);
        }
        return $this->accountNottificationEmails[$accountId];
    }

    /**
     * @return Gpf_Log_Logger
     */
    private function getLogger() {
        if ($this->logger === null) {
            $this->logger = Pap_Logger::create(Pap_Common_Constants::TYPE_ACTION);
        }
        return $this->logger;
    }

    private function setLogGroup($goupId) {
        $this->getLogger()->setGroup($goupId);
    }

    private function log($message, $logLevel = Gpf_Log::DEBUG) {
        $this->getLogger()->log($this->getName() .': '. $message, $logLevel);
    }
}
?>
