<?php
/**
 *   @copyright Copyright (c) 2007 Quality Unit s.r.o.
 *   @author Martin Pullmann
 *   @package PostAffiliatePro
 *   @since Version 1.0.0
 *
 *   Licensed under the Quality Unit, s.r.o. Standard End User License Agreement,
 *   Version 1.0 (the "License"); you may not use this file except in compliance
 *   with the License. You may obtain a copy of the License at
 *   http://www.postaffiliatepro.com/licenses/license
 *
 */

/**
 * @package PostAffiliatePro
 */
class RestCommissions_Main extends Gpf_Plugins_Handler {

    const PLUGIN_NAME = 'Rest Commissions:'; // used for debug messages
    const RESTCOMM_FORMULA = 'RestCommissionsFormula';

    private static $instance = false;

    /**
     * @return RestCommissions_Main
     */
    public static function getHandlerInstance() {
        if(!self::$instance)  {
            return self::$instance = new RestCommissions_Main();
        }
        return self::$instance;
    }

    public function initFields(Pap_Merchants_Campaign_CommissionTypeEditAdditionalForm $additionalDetails) {
        $additionalDetails->addTextBox($this->_("Rest commission formula"), self::RESTCOMM_FORMULA, $this->_('This is a formula based on which all the commissions will be counted. If main commission structure should be set to 1st tier 5%; 2nd tier 4% and 3rd tier 3% the formula would look like %3-%4-%5. In case you want to payout flat commission, the formula requires only numbers: 3-4-5. This structure defines tiers from right to left, meaning the first one from left is top tier, then their child, then grandchild etc...'));
    }

    public function save(Gpf_Rpc_Form $form) {
        $formula = $form->getFieldValue(self::RESTCOMM_FORMULA);

        $commTypeAttr = Pap_Db_Table_CommissionTypeAttributes::getInstance();
        $commTypeAttr->setCommissionTypeAttributeValue($form->getFieldValue('Id'), self::RESTCOMM_FORMULA, $formula);
    }

    public function load(Gpf_Rpc_Form $form) {
        $form->setField(self::RESTCOMM_FORMULA, $this->getFormula($form->getFieldValue('Id')));
    }

    private function getFormula($id) {
        $commTypeAttr = Pap_Db_Table_CommissionTypeAttributes::getInstance();
        try {
            return $commTypeAttr->getCommissionTypeAttribute($id, self::RESTCOMM_FORMULA)->getValue();
        } catch (Gpf_DbEngine_NoRowException $e) {
            return '';
        }
    }

    public function beforeSave(Pap_Common_TransactionCompoundContext $transactionCompoundContext) {
        if ($this->getFormula($transactionCompoundContext->getTransaction()->getCommissionTypeId()) == '') {
            Gpf_Log::debug($this->getPluginName().' no formula defined for this commission type/campaign, Rest Commission won\'t be applied.');
            return;
        }

        $transactionCompoundContext->setSaveTransaction(false); // ignore the original transaction commission
    }

    public function initSettings(Gpf_Settings_Gpf $context) {
        //just for compatibility reasons with older relases which used to call initSettings through extension point
    }

    public function modifyCommission(Pap_Common_SaveCommissionCompoundContext $compoundContext) {
        $context = $compoundContext->getContext();

        $context->debug($this->getPluginName().' plugin started.');

        if ($compoundContext->getTier() != 1) {
            $context->debug($this->getPluginName().' Transaction is not tier one, stopping.');  // ignore original tiers
            return;
        }

        $transaction = $context->getTransaction();

        $formula = $this->getFormula($transaction->getCommissionTypeId());
        if ($formula == '') {
            $context->debug($this->getPluginName().' no formula defined for this commission type/campaign, Rest Commission won\'t be applied.');
            return;
        }

        $comm_strings = explode('-',$formula); // %3-%4-%5

        foreach ($comm_strings as $key => $value) {
            $type = '';
            if (strpos($value,'%') !== false) $type = '%';
            $commissions[] = array('type' => $type, 'value' => str_replace('%','',$value));
        }

        $user = $compoundContext->getUser(); // 1st tier

        $user_string = ''; // for tier commission prediction, to correctly assign rest commissions
        $parentID = $user->getId(); // to start loading affiliates from the actual affiliate
        for ($i = 0; $i<count($commissions); $i++) {
            $user_string = '-'.$parentID.$user_string;

            $user2 = new Pap_Common_User();
            $user2->setId($parentID);
            $user2->load(); // no need for TRY-CATCH, the parent user must exist, we checked it in the previous loop

            if ($user2->getParentUserId() == '') { // we are at the top, break the loop
                break;
            }

            $parentID = $user2->getParentUserId();
        }
        $user_string = substr($user_string,1); // remove remaining dash on start

        $users = explode('-',$user_string);

        // Create commissions for each user from $users array:
        $p = 1;
        for ($i = count($commissions)-1; $i >= 0; $i--) {
            if (isset($users[$i])) {
                $this->createRestCommission($users[$i], $commissions[$i], $transaction, $p, $context);
                $p++;
                $context->debug($this->getPluginName().' commission applied.');
            }
            else {
                if ($commissions[$i-1]['type'] != $commissions[$i]['type']) {
                    $context->debug($this->getPluginName().' Commission type mismatch - trying to combine percent commission wiht flat commission. All commission types have to be the same! STOPPED');
                    break;
                }
                $commissions[$i-1]['value'] = $commissions[$i-1]['value'] + $commissions[$i]['value'];
            }
        }

        $context->debug($this->getPluginName().' plugin ended');
    }

    private function createRestCommission($userID, $commission, Pap_Common_Transaction $transaction, $tier, Pap_Contexts_Tracking $context){
        $transaction->setId('');
        $transaction->setCommission($this->computeCommission($commission, $transaction->getTotalCost()));
        $transaction->setTier($tier);
        $transaction->setUserId($userID);
        $transaction->insert();
        return;
    }

    private function computeCommission($commission, $total){ // $commission['type'].$commission['value']
        if ($commission['type'] == '') {
            return $commission['value'];
        }
        return $total * $commission['value'] / 100;
    }

    protected function getPluginName() {
        return self::PLUGIN_NAME;
    }
}

?>
